@@ -24,7 +24,7 @@ astro = astro[30:180, 150:300]
 noisy = astro + 0.3 * np.random.random(astro.shape)
 noisy = np.clip(noisy, 0, 1)
 
-denoise = denoise_nl_means(noisy, 7, 9, 0.08)
+denoise = denoise_nl_means(noisy, 7, 9, 0.08, multichannel=True)
 
 fig, ax = plt.subplots(ncols=2, figsize=(8, 4), sharex=True, sharey=True,
                        subplot_kw={'adjustable': 'box-forced'})
