@@ -3,7 +3,7 @@
 Build image pyramids
 ====================
 
-The `pyramid_gaussian` function takes an image and yields successive images
+The ``pyramid_gaussian`` function takes an image and yields successive images
 shrunk by a constant scale factor. Image pyramids are often used, e.g., to
 implement algorithms for denoising, texture discrimination, and scale-
 invariant detection.
@@ -20,7 +20,7 @@ image = data.astronaut()
 rows, cols, dim = image.shape
 pyramid = tuple(pyramid_gaussian(image, downscale=2))
 
-composite_image = np.zeros((rows, cols + cols / 2, 3), dtype=np.double)
+composite_image = np.zeros((rows, cols + cols // 2, 3), dtype=np.double)
 
 composite_image[:rows, :cols, :] = pyramid[0]
 
