@@ -14,15 +14,13 @@ def configuration(parent_package='', top_path=None):
     config.add_data_dir('tests')
 
     cython(['geometry.pyx'], working_path=base_path)
-    cython(['interpolation.pyx'], working_path=base_path)
     cython(['transform.pyx'], working_path=base_path)
+    cython(['interpolation.pyx'], working_path=base_path)
 
     config.add_extension('geometry', sources=['geometry.c'])
-    config.add_extension('interpolation', sources=['interpolation.c'],
-                         include_dirs=[get_numpy_include_dirs()])
     config.add_extension('transform', sources=['transform.c'],
                          include_dirs=[get_numpy_include_dirs()])
-
+    config.add_extension('interpolation', sources=['interpolation.c'])
     return config
 
 
@@ -30,7 +28,7 @@ if __name__ == '__main__':
     from numpy.distutils.core import setup
     setup(maintainer='scikit-image Developers',
           author='scikit-image Developers',
-          maintainer_email='scikit-image@googlegroups.com',
+          maintainer_email='scikit-image@python.org',
           description='Transforms',
           url='https://github.com/scikit-image/scikit-image',
           license='SciPy License (BSD Style)',
