@@ -1,5 +1,6 @@
 from .colorconv import (convert_colorspace,
                         guess_spatial_dimensions,
+                        rgba2rgb,
                         rgb2hsv,
                         hsv2rgb,
                         rgb2xyz,
@@ -9,6 +10,7 @@ from .colorconv import (convert_colorspace,
                         rgb2grey,
                         rgb2gray,
                         gray2rgb,
+                        grey2rgb,
                         xyz2lab,
                         lab2xyz,
                         lab2rgb,
@@ -21,6 +23,14 @@ from .colorconv import (convert_colorspace,
                         hed2rgb,
                         lab2lch,
                         lch2lab,
+                        rgb2yuv,
+                        yuv2rgb,
+                        rgb2yiq,
+                        yiq2rgb,
+                        rgb2ypbpr,
+                        ypbpr2rgb,
+                        rgb2ycbcr,
+                        ycbcr2rgb,
                         separate_stains,
                         combine_stains,
                         rgb_from_hed,
@@ -66,6 +76,7 @@ __all__ = ['convert_colorspace',
            'rgb2grey',
            'rgb2gray',
            'gray2rgb',
+           'grey2rgb',
            'xyz2lab',
            'lab2xyz',
            'lab2rgb',
@@ -74,6 +85,14 @@ __all__ = ['convert_colorspace',
            'hed2rgb',
            'lab2lch',
            'lch2lab',
+           'rgb2yuv',
+           'yuv2rgb',
+           'rgb2yiq',
+           'yiq2rgb',
+           'rgb2ypbpr',
+           'ypbpr2rgb',
+           'rgb2ycbcr',
+           'ycbcr2rgb',
            'separate_stains',
            'combine_stains',
            'rgb_from_hed',
