@@ -11,6 +11,7 @@ Authors
 :license: modified BSD
 """
 
+from __future__ import division
 import os.path
 
 import numpy as np
@@ -36,10 +37,16 @@ from skimage.color import (rgb2hsv, hsv2rgb,
                            xyz2luv, luv2xyz,
                            luv2rgb, rgb2luv,
                            lab2lch, lch2lab,
+                           rgb2yuv, yuv2rgb,
+                           rgb2yiq, yiq2rgb,
+                           rgb2ypbpr, ypbpr2rgb,
+                           rgb2ycbcr, ycbcr2rgb,
+                           rgba2rgb,
                            guess_spatial_dimensions
                            )
 
-from skimage import data_dir, data
+from skimage import data_dir
+from skimage._shared._warnings import expected_warnings
 
 import colorsys
 
@@ -61,6 +68,9 @@ class TestColorconv(TestCase):
 
     img_rgb = imread(os.path.join(data_dir, 'color.png'))
     img_grayscale = imread(os.path.join(data_dir, 'camera.png'))
+    img_rgba = np.array([[[0, 0.5, 1, 0],
+                          [0, 0.5, 1, 1],
+                          [0, 0.5, 1, 0.5]]]).astype(np.float)
 
     colbars = np.array([[1, 1, 0, 0, 1, 1, 0, 0],
                         [1, 1, 1, 1, 0, 0, 0, 0],
@@ -69,26 +79,42 @@ class TestColorconv(TestCase):
     colbars_point75 = colbars * 0.75
     colbars_point75_array = np.swapaxes(colbars_point75.reshape(3, 4, 2), 0, 2)
 
-    xyz_array = np.array([[[0.4124, 0.21260, 0.01930]],  # red
-                          [[0, 0, 0]],  # black
-                          [[.9505, 1., 1.089]],  # white
-                          [[.1805, .0722, .9505]],  # blue
-                          [[.07719, .15438, .02573]],  # green
+    xyz_array = np.array([[[0.4124, 0.21260, 0.01930]],    # red
+                          [[0, 0, 0]],    # black
+                          [[.9505, 1., 1.089]],    # white
+                          [[.1805, .0722, .9505]],    # blue
+                          [[.07719, .15438, .02573]],    # green
                           ])
-    lab_array = np.array([[[53.233, 80.109, 67.220]],  # red
-                          [[0., 0., 0.]],  # black
-                          [[100.0, 0.005, -0.010]],  # white
-                          [[32.303, 79.197, -107.864]],  # blue
-                          [[46.229, -51.7, 49.898]],  # green
+    lab_array = np.array([[[53.233, 80.109, 67.220]],    # red
+                          [[0., 0., 0.]],    # black
+                          [[100.0, 0.005, -0.010]],    # white
+                          [[32.303, 79.197, -107.864]],    # blue
+                          [[46.229, -51.7, 49.898]],    # green
                           ])
 
-    luv_array = np.array([[[53.233, 175.053, 37.751]], # red
-                          [[0., 0., 0.]], # black
-                          [[100., 0.001, -0.017]], # white
-                          [[32.303, -9.400, -130.358]], # blue
-                          [[46.228, -43.774, 56.589]], # green
+    luv_array = np.array([[[53.233, 175.053, 37.751]],   # red
+                          [[0., 0., 0.]],   # black
+                          [[100., 0.001, -0.017]],   # white
+                          [[32.303, -9.400, -130.358]],   # blue
+                          [[46.228, -43.774, 56.589]],   # green
                           ])
 
+    # RGBA to RGB
+    def test_rgba2rgb_conversion(self):
+        rgba = self.img_rgba
+        rgb = rgba2rgb(rgba)
+        expected = np.array([[[1, 1, 1],
+                              [0, 0.5, 1],
+                              [0.5, 0.75, 1]]]).astype(np.float)
+        self.assertEqual(rgb.shape, expected.shape)
+        assert_almost_equal(rgb, expected)
+
+    def test_rgba2rgb_error_grayscale(self):
+        self.assertRaises(ValueError, rgba2rgb, self.img_grayscale)
+
+    def test_rgba2rgb_error_rgb(self):
+        self.assertRaises(ValueError, rgba2rgb, self.img_rgb)
+
     # RGB to HSV
     def test_rgb2hsv_conversion(self):
         rgb = img_as_float(self.img_rgb)[::16, ::16]
@@ -154,7 +180,9 @@ class TestColorconv(TestCase):
     # RGB<->HED roundtrip with ubyte image
     def test_hed_rgb_roundtrip(self):
         img_rgb = img_as_ubyte(self.img_rgb)
-        assert_equal(img_as_ubyte(hed2rgb(rgb2hed(img_rgb))), img_rgb)
+        with expected_warnings(['precision loss']):
+            new = img_as_ubyte(hed2rgb(rgb2hed(img_rgb)))
+        assert_equal(new, img_rgb)
 
     # RGB<->HED roundtrip with float image
     def test_hed_rgb_float_roundtrip(self):
@@ -222,19 +250,72 @@ class TestColorconv(TestCase):
 
         assert_equal(g.shape, (1, 1))
 
-    def test_rgb2grey_on_grey(self):
-        rgb2grey(np.random.random((5, 5)))
+    def test_rgb2grey_contiguous(self):
+        x = np.random.rand(10, 10, 3)
+        assert rgb2grey(x).flags["C_CONTIGUOUS"]
+        assert rgb2grey(x[:5, :5]).flags["C_CONTIGUOUS"]
 
-    # test matrices for xyz2lab and lab2xyz generated using http://www.easyrgb.com/index.php?X=CALC
+    def test_rgb2grey_alpha(self):
+        x = np.random.rand(10, 10, 4)
+        assert rgb2grey(x).ndim == 2
+
+    def test_rgb2grey_on_grey(self):
+        rgb2grey(np.random.rand(5, 5))
+
+    # test matrices for xyz2lab and lab2xyz generated using
+    # http://www.easyrgb.com/index.php?X=CALC
     # Note: easyrgb website displays xyz*100
     def test_xyz2lab(self):
         assert_array_almost_equal(xyz2lab(self.xyz_array),
                                   self.lab_array, decimal=3)
 
+        # Test the conversion with the rest of the illuminants.
+        for I in ["d50", "d55", "d65", "d75"]:
+            for obs in ["2", "10"]:
+                fname = "lab_array_{0}_{1}.npy".format(I, obs)
+                lab_array_I_obs = np.load(
+                    os.path.join(os.path.dirname(__file__), 'data', fname))
+                assert_array_almost_equal(lab_array_I_obs,
+                                          xyz2lab(self.xyz_array, I, obs),
+                                          decimal=2)
+        for I in ["a", "e"]:
+            fname = "lab_array_{0}_2.npy".format(I)
+            lab_array_I_obs = np.load(
+                os.path.join(os.path.dirname(__file__), 'data', fname))
+            assert_array_almost_equal(lab_array_I_obs,
+                                      xyz2lab(self.xyz_array, I, "2"),
+                                      decimal=2)
+
     def test_lab2xyz(self):
         assert_array_almost_equal(lab2xyz(self.lab_array),
                                   self.xyz_array, decimal=3)
 
+        # Test the conversion with the rest of the illuminants.
+        for I in ["d50", "d55", "d65", "d75"]:
+            for obs in ["2", "10"]:
+                fname = "lab_array_{0}_{1}.npy".format(I, obs)
+                lab_array_I_obs = np.load(
+                    os.path.join(os.path.dirname(__file__), 'data', fname))
+                assert_array_almost_equal(lab2xyz(lab_array_I_obs, I, obs),
+                                          self.xyz_array, decimal=3)
+        for I in ["a", "e"]:
+            fname = "lab_array_{0}_2.npy".format(I, obs)
+            lab_array_I_obs = np.load(
+                os.path.join(os.path.dirname(__file__), 'data', fname))
+            assert_array_almost_equal(lab2xyz(lab_array_I_obs, I, "2"),
+                                      self.xyz_array, decimal=3)
+
+        # And we include a call to test the exception handling in the code.
+        try:
+            xs = lab2xyz(lab_array_I_obs, "NaI", "2")   # Not an illuminant
+        except ValueError:
+            pass
+
+        try:
+            xs = lab2xyz(lab_array_I_obs, "d50", "42")   # Not a degree
+        except ValueError:
+            pass
+
     def test_rgb2lab_brucelindbloom(self):
         """
         Test the RGB->Lab conversion by comparing to the calculator on the
@@ -265,10 +346,42 @@ class TestColorconv(TestCase):
         assert_array_almost_equal(xyz2luv(self.xyz_array),
                                   self.luv_array, decimal=3)
 
+        # Test the conversion with the rest of the illuminants.
+        for I in ["d50", "d55", "d65", "d75"]:
+            for obs in ["2", "10"]:
+                fname = "luv_array_{0}_{1}.npy".format(I, obs)
+                luv_array_I_obs = np.load(
+                    os.path.join(os.path.dirname(__file__), 'data', fname))
+                assert_array_almost_equal(luv_array_I_obs,
+                                          xyz2luv(self.xyz_array, I, obs),
+                                          decimal=2)
+        for I in ["a", "e"]:
+            fname = "luv_array_{0}_2.npy".format(I)
+            luv_array_I_obs = np.load(
+                os.path.join(os.path.dirname(__file__), 'data', fname))
+            assert_array_almost_equal(luv_array_I_obs,
+                                      xyz2luv(self.xyz_array, I, "2"),
+                                      decimal=2)
+
     def test_luv2xyz(self):
         assert_array_almost_equal(luv2xyz(self.luv_array),
                                   self.xyz_array, decimal=3)
 
+        # Test the conversion with the rest of the illuminants.
+        for I in ["d50", "d55", "d65", "d75"]:
+            for obs in ["2", "10"]:
+                fname = "luv_array_{0}_{1}.npy".format(I, obs)
+                luv_array_I_obs = np.load(
+                    os.path.join(os.path.dirname(__file__), 'data', fname))
+                assert_array_almost_equal(luv2xyz(luv_array_I_obs, I, obs),
+                                          self.xyz_array, decimal=3)
+        for I in ["a", "e"]:
+            fname = "luv_array_{0}_2.npy".format(I, obs)
+            luv_array_I_obs = np.load(
+                os.path.join(os.path.dirname(__file__), 'data', fname))
+            assert_array_almost_equal(luv2xyz(luv_array_I_obs, I, "2"),
+                                      self.xyz_array, decimal=3)
+
     def test_rgb2luv_brucelindbloom(self):
         """
         Test the RGB->Lab conversion by comparing to the calculator on the
@@ -293,6 +406,26 @@ class TestColorconv(TestCase):
         img_rgb = img_as_float(self.img_rgb)
         assert_array_almost_equal(luv2rgb(rgb2luv(img_rgb)), img_rgb)
 
+    def test_lab_rgb_outlier(self):
+        lab_array = np.ones((3, 1, 3))
+        lab_array[0] = [50, -12, 85]
+        lab_array[1] = [50, 12, -85]
+        lab_array[2] = [90, -4, -47]
+        rgb_array = np.array([[[0.501, 0.481, 0]],
+                              [[0, 0.482, 1.]],
+                              [[0.578, 0.914, 1.]],
+                              ])
+        assert_almost_equal(lab2rgb(lab_array), rgb_array, decimal=3)
+
+    def test_lab_full_gamut(self):
+        a, b = np.meshgrid(np.arange(-100, 100), np.arange(-100, 100))
+        L = np.ones(a.shape)
+        lab = np.dstack((L, a, b))
+        for value in [0, 10, 20]:
+            lab[:, :, 0] = value
+            with expected_warnings(['Color data out of range']):
+                lab2xyz(lab)
+
     def test_lab_lch_roundtrip(self):
         rgb = img_as_float(self.img_rgb)
         lab = rgb2lab(rgb)
@@ -329,6 +462,33 @@ class TestColorconv(TestCase):
         rgb = img_as_float(self.img_rgb[:1, :1, :])
         return rgb2lab(rgb)[0, 0, :]
 
+    def test_yuv(self):
+        rgb = np.array([[[1.0, 1.0, 1.0]]])
+        assert_array_almost_equal(rgb2yuv(rgb), np.array([[[1, 0, 0]]]))
+        assert_array_almost_equal(rgb2yiq(rgb), np.array([[[1, 0, 0]]]))
+        assert_array_almost_equal(rgb2ypbpr(rgb), np.array([[[1, 0, 0]]]))
+        assert_array_almost_equal(rgb2ycbcr(rgb), np.array([[[235, 128, 128]]]))
+        rgb = np.array([[[0.0, 1.0, 0.0]]])
+        assert_array_almost_equal(rgb2yuv(rgb), np.array([[[0.587, -0.28886916, -0.51496512]]]))
+        assert_array_almost_equal(rgb2yiq(rgb), np.array([[[0.587, -0.27455667, -0.52273617]]]))
+        assert_array_almost_equal(rgb2ypbpr(rgb), np.array([[[0.587, -0.331264, -0.418688]]]))
+        assert_array_almost_equal(rgb2ycbcr(rgb), np.array([[[144.553,   53.797,   34.214]]]))
+
+    def test_yuv_roundtrip(self):
+        img_rgb = img_as_float(self.img_rgb)[::16, ::16]
+        assert_array_almost_equal(yuv2rgb(rgb2yuv(img_rgb)), img_rgb)
+        assert_array_almost_equal(yiq2rgb(rgb2yiq(img_rgb)), img_rgb)
+        assert_array_almost_equal(ypbpr2rgb(rgb2ypbpr(img_rgb)), img_rgb)
+        assert_array_almost_equal(ycbcr2rgb(rgb2ycbcr(img_rgb)), img_rgb)
+
+    def test_rgb2yiq_conversion(self):
+        rgb = img_as_float(self.img_rgb)[::16, ::16]
+        yiq = rgb2yiq(rgb).reshape(-1, 3)
+        gt = np.array([colorsys.rgb_to_yiq(pt[0], pt[1], pt[2])
+                       for pt in rgb.reshape(-1, 3)]
+                      )
+        assert_almost_equal(yiq, gt, decimal=2)
+
 
 def test_gray2rgb():
     x = np.array([0, 0.5, 1])
@@ -339,6 +499,8 @@ def test_gray2rgb():
 
     assert_equal(y.shape, (3, 1, 3))
     assert_equal(y.dtype, x.dtype)
+    assert_equal(y[..., 0], x)
+    assert_equal(y[0, 0, :], [0, 0, 0])
 
     x = np.array([[0, 128, 255]], dtype=np.uint8)
     z = gray2rgb(x)
@@ -349,11 +511,28 @@ def test_gray2rgb():
 
 
 def test_gray2rgb_rgb():
-    x = np.random.random((5, 5, 4))
+    x = np.random.rand(5, 5, 4)
     y = gray2rgb(x)
     assert_equal(x, y)
 
 
+def test_gray2rgb_alpha():
+    x = np.random.random((5, 5, 4))
+    assert_equal(gray2rgb(x, alpha=None).shape, (5, 5, 4))
+    assert_equal(gray2rgb(x, alpha=False).shape, (5, 5, 3))
+    assert_equal(gray2rgb(x, alpha=True).shape, (5, 5, 4))
+
+    x = np.random.random((5, 5, 3))
+    assert_equal(gray2rgb(x, alpha=None).shape, (5, 5, 3))
+    assert_equal(gray2rgb(x, alpha=False).shape, (5, 5, 3))
+    assert_equal(gray2rgb(x, alpha=True).shape, (5, 5, 4))
+
+    assert_equal(gray2rgb(np.array([[1, 2], [3, 4.]]),
+                          alpha=True)[0, 0, 3], 1)
+    assert_equal(gray2rgb(np.array([[1, 2], [3, 4]], dtype=np.uint8),
+                          alpha=True)[0, 0, 3], 255)
+
+
 if __name__ == "__main__":
     from numpy.testing import run_module_suite
     run_module_suite()
