@@ -3,7 +3,6 @@ import itertools
 import numpy as np
 from numpy import testing
 from skimage.color.colorlabel import label2rgb
-from skimage._shared._warnings import expected_warnings
 from numpy.testing import (assert_array_almost_equal as assert_close,
                            assert_array_equal, assert_warns)
 
@@ -70,11 +69,21 @@ def test_bg_and_color_cycle():
     for pixel, color in zip(rgb[0, 1:], itertools.cycle(colors)):
         assert_close(pixel, color)
 
+def test_negative_labels():
+    labels = np.array([0, -1, -2, 0])
+    rout = np.array([(0., 0., 0.), (0., 0., 1.), (1., 0., 0.), (0., 0., 0.)])
+    assert_close(rout, label2rgb(labels, bg_label=0, alpha=1, image_alpha=1))
+
+def test_nonconsecutive():
+    labels = np.array([0, 2, 4, 0])
+    colors=[(1, 0, 0), (0, 0, 1)]
+    rout = np.array([(1., 0., 0.), (0., 0., 1.), (1., 0., 0.), (1., 0., 0.)])
+    assert_close(rout, label2rgb(labels, colors=colors, alpha=1, image_alpha=1))
 
 def test_label_consistency():
     """Assert that the same labels map to the same colors."""
     label_1 = np.arange(5).reshape(1, -1)
-    label_2 = np.array([2, 4])
+    label_2 = np.array([0, 1])
     colors = [(1, 0, 0), (0, 1, 0), (0, 0, 1), (1, 1, 0), (1, 0, 1)]
     # Set alphas just in case the defaults change
     rgb_1 = label2rgb(label_1, colors=colors)
