@@ -1,3 +1,5 @@
+# coding: utf-8
+
 """Standard test images.
 
 For more images, see
@@ -8,36 +10,42 @@ For more images, see
 
 import os as _os
 
+import numpy as np
+
 from .. import data_dir
 from ..io import imread, use_plugin
-from .._shared.utils import deprecated
+from .._shared._warnings import expected_warnings
 from ._binary_blobs import binary_blobs
+from .. import img_as_bool
 
 __all__ = ['load',
+           'astronaut',
            'camera',
-           'lena',
-           'text',
            'checkerboard',
+           'chelsea',
+           'clock',
+           'coffee',
            'coins',
+           'horse',
+           'hubble_deep_field',
+           'immunohistochemistry',
+           'logo',
            'moon',
            'page',
-           'horse',
-           'clock',
-           'immunohistochemistry',
-           'chelsea',
-           'coffee',
-           'hubble_deep_field',
+           'text',
            'rocket',
-           'astronaut']
+           'stereo_motorcycle']
 
 
-def load(f):
+def load(f, as_grey=False):
     """Load an image file located in the data directory.
 
     Parameters
     ----------
     f : string
         File name.
+    as_grey : bool, optional
+        Convert to greyscale.
 
     Returns
     -------
@@ -45,7 +53,7 @@ def load(f):
         Image loaded from ``skimage.data_dir``.
     """
     use_plugin('pil')
-    return imread(_os.path.join(data_dir, f))
+    return imread(_os.path.join(data_dir, f), as_grey=as_grey)
 
 
 def camera():
@@ -53,26 +61,14 @@ def camera():
 
     Often used for segmentation and denoising examples.
 
+    Returns
+    -------
+    camera : (512, 512) uint8 ndarray
+        Camera image.
     """
     return load("camera.png")
 
 
-@deprecated('skimage.data.astronaut')
-def lena():
-    """Colour "Lena" image.
-
-    **This image has been removed from scikit-image due to copyright
-    concerns.**
-
-    The standard, yet sometimes controversial Lena test image was
-    scanned from the November 1972 edition of Playboy magazine.  From
-    an image processing perspective, this image is useful because it
-    contains smooth, textured, shaded as well as detail areas.
-
-    """
-    raise RuntimeError("This image has been removed due to copyright concerns.")
-
-
 def astronaut():
     """Colour image of the astronaut Eileen Collins.
 
@@ -82,10 +78,14 @@ def astronaut():
     and 10 minutes in outer space.
 
     This image was downloaded from the NASA Great Images database
-    <http://grin.hq.nasa.gov/ABSTRACTS/GPN-2000-001177.html>`__.
+    <https://flic.kr/p/r9qvLn>`__.
 
     No known copyright restrictions, released into the public domain.
 
+    Returns
+    -------
+    astronaut : (512, 512, 3) uint8 ndarray
+        Astronaut image.
     """
 
     return load("astronaut.png")
@@ -101,6 +101,10 @@ def text():
 
     No known copyright restrictions, released into the public domain.
 
+    Returns
+    -------
+    text : (172, 448) uint8 ndarray
+        Text image.
     """
 
     return load("text.png")
@@ -113,6 +117,10 @@ def checkerboard():
     corner-points are easy to locate.  Because of the many parallel
     edges, they also visualise distortions particularly well.
 
+    Returns
+    -------
+    checkerboard : (200, 200) uint8 ndarray
+        Checkerboard image.
     """
     return load("chessboard_GRAY.png")
 
@@ -130,20 +138,39 @@ def coins():
     -----
     This image was downloaded from the
     `Brooklyn Museum Collection
-    <http://www.brooklynmuseum.org/opencollection/archives/image/617/image>`__.
+    <http://www.brooklynmuseum.org/opencollection/archives/image/33814>`__.
 
     No known copyright restrictions.
 
+    Returns
+    -------
+    coins : (303, 384) uint8 ndarray
+        Coins image.
     """
     return load("coins.png")
 
 
+def logo():
+    """Scikit-image logo, a RGBA image.
+
+    Returns
+    -------
+    logo : (500, 500, 4) uint8 ndarray
+        Logo image.
+    """
+    return load("logo.png")
+
+
 def moon():
     """Surface of the moon.
 
     This low-contrast image of the surface of the moon is useful for
     illustrating histogram equalization and contrast stretching.
 
+    Returns
+    -------
+    moon : (512, 512) uint8 ndarray
+        Moon image.
     """
     return load("moon.png")
 
@@ -154,6 +181,10 @@ def page():
     This image of printed text is useful for demonstrations requiring uneven
     background illumination.
 
+    Returns
+    -------
+    page : (191, 384) uint8 ndarray
+        Page image.
     """
     return load("page.png")
 
@@ -167,8 +198,13 @@ def horse():
     Released into public domain and drawn and uploaded by Andreas Preuss
     (marauder).
 
+    Returns
+    -------
+    horse : (328, 400) bool ndarray
+        Horse image.
     """
-    return load("horse.png")
+    with expected_warnings(['Possible precision loss', 'Possible sign loss']):
+        return img_as_bool(load("horse.png", as_grey=True))
 
 
 def clock():
@@ -180,6 +216,10 @@ def clock():
 
     Released into the public domain by the photographer (Stefan van der Walt).
 
+    Returns
+    -------
+    clock : (300, 400) uint8 ndarray
+        Clock image.
     """
     return load("clock_motion.png")
 
@@ -196,6 +236,10 @@ def immunohistochemistry():
 
     No known copyright restrictions.
 
+    Returns
+    -------
+    immunohistochemistry : (512, 512, 3) uint8 ndarray
+        Immunohistochemistry image.
     """
     return load("ihc.png")
 
@@ -210,6 +254,10 @@ def chelsea():
     -----
     No copyright restrictions.  CC0 by the photographer (Stefan van der Walt).
 
+    Returns
+    -------
+    chelsea : (300, 451, 3) uint8 ndarray
+        Chelsea image.
     """
     return load("chelsea.png")
 
@@ -225,6 +273,10 @@ def coffee():
     -----
     No copyright restrictions.  CC0 by the photographer (Rachel Michetti).
 
+    Returns
+    -------
+    coffee : (400, 600, 3) uint8 ndarray
+        Coffee image.
     """
     return load("coffee.png")
 
@@ -242,9 +294,13 @@ def hubble_deep_field():
     `HubbleSite
     <http://hubblesite.org/newscenter/archive/releases/2012/37/image/a/>`__.
 
-    The image was captured by NASA and `may be freely used in the
-    public domain <http://www.nasa.gov/audience/formedia/features/MP_Photo_Guidelines.html>`_.
+    The image was captured by NASA and `may be freely used in the public domain
+    <http://www.nasa.gov/audience/formedia/features/MP_Photo_Guidelines.html>`_.
 
+    Returns
+    -------
+    hubble_deep_field : (872, 1000, 3) uint8 ndarray
+        Hubble deep field image.
     """
     return load("hubble_deep_field.jpg")
 
@@ -264,5 +320,67 @@ def rocket():
     The image was captured by SpaceX and `released in the public domain
     <http://arstechnica.com/tech-policy/2015/03/elon-musk-puts-spacex-photos-into-the-public-domain/>`_.
 
+    Returns
+    -------
+    rocket : (427, 640, 3) uint8 ndarray
+        Rocket image.
     """
     return load("rocket.jpg")
+
+
+def stereo_motorcycle():
+    """Rectified stereo image pair with ground-truth disparities.
+
+    The two images are rectified such that every pixel in the left image has its
+    corresponding pixel on the same scanline in the right image. That means that
+    both images are warped such that they have the same orientation but a
+    horizontal spatial offset (baseline). The ground-truth pixel offset in
+    column direction is specified by the included disparity map.
+
+    The two images are part of the Middlebury 2014 stereo benchmark. The dataset
+    was created by Nera Nesic, Porter Westling, Xi Wang, York Kitajima, Greg
+    Krathwohl, and Daniel Scharstein at Middlebury College. A detailed
+    description of the acquisition process can be found in [1]_.
+
+    The images included here are down-sampled versions of the default exposure
+    images in the benchmark. The images are down-sampled by a factor of 4 using
+    the function `skimage.transform.downscale_local_mean`. The calibration data
+    in the following and the included ground-truth disparity map are valid for
+    the down-sampled images::
+
+        Focal length:           994.978px
+        Principal point x:      311.193px
+        Principal point y:      254.877px
+        Principal point dx:      31.086px
+        Baseline:               193.001mm
+
+    Returns
+    -------
+    img_left : (500, 741, 3) uint8 ndarray
+        Left stereo image.
+    img_right : (500, 741, 3) uint8 ndarray
+        Right stereo image.
+    disp : (500, 741, 3) float ndarray
+        Ground-truth disparity map, where each value describes the offset in
+        column direction between corresponding pixels in the left and the right
+        stereo images. E.g. the corresponding pixel of
+        ``img_left[10, 10 + disp[10, 10]]`` is ``img_right[10, 10]``.
+        NaNs denote pixels in the left image that do not have ground-truth.
+
+    Notes
+    -----
+    The original resolution images, images with different exposure and lighting,
+    and ground-truth depth maps can be found at the Middlebury website [2]_.
+
+    References
+    ----------
+    .. [1] D. Scharstein, H. Hirschmueller, Y. Kitajima, G. Krathwohl, N. Nesic,
+           X. Wang, and P. Westling. High-resolution stereo datasets with
+           subpixel-accurate ground truth. In German Conference on Pattern
+           Recognition (GCPR 2014), Muenster, Germany, September 2014.
+    .. [2] http://vision.middlebury.edu/stereo/data/scenes2014/
+
+    """
+    return (load("motorcycle_left.png"),
+            load("motorcycle_right.png"),
+            np.load(_os.path.join(data_dir, "motorcycle_disp.npz"))["arr_0"])
