@@ -1,9 +1,8 @@
 import warnings
 
 import numpy as np
-from numpy.testing import assert_array_almost_equal as assert_close
-from numpy.testing import (assert_array_equal, assert_raises,
-                           assert_almost_equal)
+from numpy.testing import (assert_array_equal, assert_array_almost_equal,
+                           assert_raises, assert_almost_equal)
 
 import skimage
 from skimage import data
@@ -45,6 +44,7 @@ test_img_int = data.camera()
 test_img = skimage.img_as_float(test_img_int)
 test_img = exposure.rescale_intensity(test_img / 5. + 100)
 
+
 def test_equalize_uint8_approx():
     """Check integer bins used for uint8 images."""
     img_eq0 = exposure.equalize_hist(test_img_int)
@@ -122,7 +122,6 @@ def test_intensity_range_clipped_float():
 # Test rescale intensity
 # ======================
 
-
 uint10_max = 2**10 - 1
 uint12_max = 2**12 - 1
 uint14_max = 2**14 - 1
@@ -133,56 +132,56 @@ def test_rescale_stretch():
     image = np.array([51, 102, 153], dtype=np.uint8)
     out = exposure.rescale_intensity(image)
     assert out.dtype == np.uint8
-    assert_close(out, [0, 127, 255])
+    assert_array_almost_equal(out, [0, 127, 255])
 
 
 def test_rescale_shrink():
     image = np.array([51., 102., 153.])
     out = exposure.rescale_intensity(image)
-    assert_close(out, [0, 0.5, 1])
+    assert_array_almost_equal(out, [0, 0.5, 1])
 
 
 def test_rescale_in_range():
     image = np.array([51., 102., 153.])
     out = exposure.rescale_intensity(image, in_range=(0, 255))
-    assert_close(out, [0.2, 0.4, 0.6])
+    assert_array_almost_equal(out, [0.2, 0.4, 0.6])
 
 
 def test_rescale_in_range_clip():
     image = np.array([51., 102., 153.])
     out = exposure.rescale_intensity(image, in_range=(0, 102))
-    assert_close(out, [0.5, 1, 1])
+    assert_array_almost_equal(out, [0.5, 1, 1])
 
 
 def test_rescale_out_range():
     image = np.array([-10, 0, 10], dtype=np.int8)
     out = exposure.rescale_intensity(image, out_range=(0, 127))
     assert out.dtype == np.int8
-    assert_close(out, [0, 63, 127])
+    assert_array_almost_equal(out, [0, 63, 127])
 
 
 def test_rescale_named_in_range():
     image = np.array([0, uint10_max, uint10_max + 100], dtype=np.uint16)
     out = exposure.rescale_intensity(image, in_range='uint10')
-    assert_close(out, [0, uint16_max, uint16_max])
+    assert_array_almost_equal(out, [0, uint16_max, uint16_max])
 
 
 def test_rescale_named_out_range():
     image = np.array([0, uint16_max], dtype=np.uint16)
     out = exposure.rescale_intensity(image, out_range='uint10')
-    assert_close(out, [0, uint10_max])
+    assert_array_almost_equal(out, [0, uint10_max])
 
 
 def test_rescale_uint12_limits():
     image = np.array([0, uint16_max], dtype=np.uint16)
     out = exposure.rescale_intensity(image, out_range='uint12')
-    assert_close(out, [0, uint12_max])
+    assert_array_almost_equal(out, [0, uint12_max])
 
 
 def test_rescale_uint14_limits():
     image = np.array([0, uint16_max], dtype=np.uint16)
     out = exposure.rescale_intensity(image, out_range='uint14')
-    assert_close(out, [0, uint14_max])
+    assert_array_almost_equal(out, [0, uint14_max])
 
 
 # Test adaptive histogram equalization
@@ -198,11 +197,9 @@ def test_adapthist_scalar():
     assert img.shape == adapted.shape
     full_scale = skimage.exposure.rescale_intensity(skimage.img_as_float(img))
 
-    assert_almost_equal = np.testing.assert_almost_equal
     assert_almost_equal(peak_snr(full_scale, adapted), 102.066, 3)
     assert_almost_equal(norm_brightness_err(full_scale, adapted),
                         0.038, 3)
-    return img, adapted
 
 
 def test_adapthist_grayscale():
@@ -211,17 +208,12 @@ def test_adapthist_grayscale():
     img = skimage.img_as_float(data.astronaut())
     img = rgb2gray(img)
     img = np.dstack((img, img, img))
-    with expected_warnings(['precision loss|non-contiguous input',
-                            'deprecated']):
-        adapted_old = exposure.equalize_adapthist(img, 10, 9, clip_limit=0.001,
-                                                  nbins=128)
     with expected_warnings(['precision loss|non-contiguous input']):
         adapted = exposure.equalize_adapthist(img, kernel_size=(57, 51),
                                               clip_limit=0.01, nbins=128)
     assert img.shape == adapted.shape
-    assert_almost_equal(peak_snr(img, adapted),  102.078, 3)
+    assert_almost_equal(peak_snr(img, adapted), 102.078, 3)
     assert_almost_equal(norm_brightness_err(img, adapted), 0.0529, 3)
-    return data, adapted
 
 
 def test_adapthist_color():
@@ -235,7 +227,6 @@ def test_adapthist_color():
     with expected_warnings(['precision loss']):
         adapted = exposure.equalize_adapthist(img, clip_limit=0.01)
 
-    assert_almost_equal = np.testing.assert_almost_equal
     assert adapted.min() == 0
     assert adapted.max() == 1.0
     assert img.shape == adapted.shape
@@ -257,11 +248,18 @@ def test_adapthist_alpha():
     img = img[:, :, :3]
     full_scale = skimage.exposure.rescale_intensity(img)
     assert img.shape == adapted.shape
-    assert_almost_equal = np.testing.assert_almost_equal
     assert_almost_equal(peak_snr(full_scale, adapted), 109.393, 2)
     assert_almost_equal(norm_brightness_err(full_scale, adapted), 0.0248, 3)
 
 
+def test_adapthist_ntiles_raises():
+    img = skimage.img_as_ubyte(data.moon())
+    assert_raises(ValueError, exposure.equalize_adapthist, img, ntiles_x=8)
+    assert_raises(ValueError, exposure.equalize_adapthist, img, ntiles_y=8)
+    assert_raises(ValueError, exposure.equalize_adapthist, img,
+                  ntiles_x=8, ntiles_y=8)
+
+
 def peak_snr(img1, img2):
     """Peak signal to noise ratio of two images
 
@@ -304,10 +302,10 @@ def norm_brightness_err(img1, img2):
     return nbe
 
 
-
 # Test Gamma Correction
 # =====================
 
+
 def test_adjust_gamma_one():
     """Same image should be returned for gamma equal to one"""
     image = np.random.uniform(0, 255, (8, 8))
@@ -326,8 +324,9 @@ def test_adjust_gamma_zero():
 def test_adjust_gamma_less_one():
     """Verifying the output with expected results for gamma
     correction with gamma equal to half"""
-    image = np.arange(0, 255, 4, np.uint8).reshape(8,8)
-    expected = np.array([[  0,  31,  45,  55,  63,  71,  78,  84],
+    image = np.arange(0, 255, 4, np.uint8).reshape((8, 8))
+    expected = np.array([
+        [  0,  31,  45,  55,  63,  71,  78,  84],
         [ 90,  95, 100, 105, 110, 115, 119, 123],
         [127, 131, 135, 139, 142, 146, 149, 153],
         [156, 159, 162, 165, 168, 171, 174, 177],
@@ -343,8 +342,9 @@ def test_adjust_gamma_less_one():
 def test_adjust_gamma_greater_one():
     """Verifying the output with expected results for gamma
     correction with gamma equal to two"""
-    image = np.arange(0, 255, 4, np.uint8).reshape(8,8)
-    expected = np.array([[  0,   0,   0,   0,   1,   1,   2,   3],
+    image = np.arange(0, 255, 4, np.uint8).reshape((8, 8))
+    expected = np.array([
+        [  0,   0,   0,   0,   1,   1,   2,   3],
         [  4,   5,   6,   7,   9,  10,  12,  14],
         [ 16,  18,  20,  22,  25,  27,  30,  33],
         [ 36,  39,  42,  45,  49,  52,  56,  60],
@@ -358,7 +358,7 @@ def test_adjust_gamma_greater_one():
 
 
 def test_adjust_gamma_neggative():
-    image = np.arange(0, 255, 4, np.uint8).reshape(8,8)
+    image = np.arange(0, 255, 4, np.uint8).reshape((8, 8))
     assert_raises(ValueError, exposure.adjust_gamma, image, -1)
 
 
@@ -368,8 +368,9 @@ def test_adjust_gamma_neggative():
 def test_adjust_log():
     """Verifying the output with expected results for logarithmic
     correction with multiplier constant multiplier equal to unity"""
-    image = np.arange(0, 255, 4, np.uint8).reshape(8,8)
-    expected = np.array([[  0,   5,  11,  16,  22,  27,  33,  38],
+    image = np.arange(0, 255, 4, np.uint8).reshape((8, 8))
+    expected = np.array([
+        [  0,   5,  11,  16,  22,  27,  33,  38],
         [ 43,  48,  53,  58,  63,  68,  73,  77],
         [ 82,  86,  91,  95, 100, 104, 109, 113],
         [117, 121, 125, 129, 133, 137, 141, 145],
@@ -385,8 +386,9 @@ def test_adjust_log():
 def test_adjust_inv_log():
     """Verifying the output with expected results for inverse logarithmic
     correction with multiplier constant multiplier equal to unity"""
-    image = np.arange(0, 255, 4, np.uint8).reshape(8,8)
-    expected = np.array([[  0,   2,   5,   8,  11,  14,  17,  20],
+    image = np.arange(0, 255, 4, np.uint8).reshape((8, 8))
+    expected = np.array([
+        [  0,   2,   5,   8,  11,  14,  17,  20],
         [ 23,  26,  29,  32,  35,  38,  41,  45],
         [ 48,  51,  55,  58,  61,  65,  68,  72],
         [ 76,  79,  83,  87,  90,  94,  98, 102],
@@ -405,8 +407,9 @@ def test_adjust_inv_log():
 def test_adjust_sigmoid_cutoff_one():
     """Verifying the output with expected results for sigmoid correction
     with cutoff equal to one and gain of 5"""
-    image = np.arange(0, 255, 4, np.uint8).reshape(8,8)
-    expected = np.array([[  1,   1,   1,   2,   2,   2,   2,   2],
+    image = np.arange(0, 255, 4, np.uint8).reshape((8, 8))
+    expected = np.array([
+        [  1,   1,   1,   2,   2,   2,   2,   2],
         [  3,   3,   3,   4,   4,   4,   5,   5],
         [  5,   6,   6,   7,   7,   8,   9,  10],
         [ 10,  11,  12,  13,  14,  15,  16,  18],
@@ -422,8 +425,9 @@ def test_adjust_sigmoid_cutoff_one():
 def test_adjust_sigmoid_cutoff_zero():
     """Verifying the output with expected results for sigmoid correction
     with cutoff equal to zero and gain of 10"""
-    image = np.arange(0, 255, 4, np.uint8).reshape(8,8)
-    expected = np.array([[127, 137, 147, 156, 166, 175, 183, 191],
+    image = np.arange(0, 255, 4, np.uint8).reshape((8, 8))
+    expected = np.array([
+        [127, 137, 147, 156, 166, 175, 183, 191],
         [198, 205, 211, 216, 221, 225, 229, 232],
         [235, 238, 240, 242, 244, 245, 247, 248],
         [249, 250, 250, 251, 251, 252, 252, 253],
@@ -439,8 +443,9 @@ def test_adjust_sigmoid_cutoff_zero():
 def test_adjust_sigmoid_cutoff_half():
     """Verifying the output with expected results for sigmoid correction
     with cutoff equal to half and gain of 10"""
-    image = np.arange(0, 255, 4, np.uint8).reshape(8,8)
-    expected = np.array([[  1,   1,   2,   2,   3,   3,   4,   5],
+    image = np.arange(0, 255, 4, np.uint8).reshape((8, 8))
+    expected = np.array([
+        [  1,   1,   2,   2,   3,   3,   4,   5],
         [  5,   6,   7,   9,  10,  12,  14,  16],
         [ 19,  22,  25,  29,  34,  39,  44,  50],
         [ 57,  64,  72,  80,  89,  99, 108, 118],
@@ -456,8 +461,9 @@ def test_adjust_sigmoid_cutoff_half():
 def test_adjust_inv_sigmoid_cutoff_half():
     """Verifying the output with expected results for inverse sigmoid
     correction with cutoff equal to half and gain of 10"""
-    image = np.arange(0, 255, 4, np.uint8).reshape(8,8)
-    expected = np.array([[253, 253, 252, 252, 251, 251, 250, 249],
+    image = np.arange(0, 255, 4, np.uint8).reshape((8, 8))
+    expected = np.array([
+        [253, 253, 252, 252, 251, 251, 250, 249],
         [249, 248, 247, 245, 244, 242, 240, 238],
         [235, 232, 229, 225, 220, 215, 210, 204],
         [197, 190, 182, 174, 165, 155, 146, 136],
@@ -471,7 +477,7 @@ def test_adjust_inv_sigmoid_cutoff_half():
 
 
 def test_negative():
-    image = np.arange(-10, 245, 4).reshape(8, 8).astype(np.double)
+    image = np.arange(-10, 245, 4).reshape((8, 8)).astype(np.double)
     assert_raises(ValueError, exposure.adjust_gamma, image)
 
 
