@@ -1,15 +1,18 @@
+from itertools import combinations_with_replacement
+
 import numpy as np
-from scipy import ndimage
+from scipy import ndimage as ndi
 from scipy import stats
 
-from skimage.util import img_as_float, pad
-from skimage.feature import peak_local_max
-from skimage.feature.util import _prepare_grayscale_input_2D
-from skimage.feature.corner_cy import _corner_fast
+from ..util import img_as_float
+from ..feature import peak_local_max
+from ..feature.util import _prepare_grayscale_input_2D
+from ..feature.corner_cy import _corner_fast
 from ._hessian_det_appx import _hessian_matrix_det
 from ..transform import integral_image
 from .._shared.utils import safe_as_int
-
+from .corner_cy import _corner_moravec, _corner_orientations
+from warnings import warn
 
 def _compute_derivatives(image, mode='constant', cval=0):
     """Compute derivatives in x and y direction using the Sobel operator.
@@ -33,8 +36,8 @@ def _compute_derivatives(image, mode='constant', cval=0):
 
     """
 
-    imy = ndimage.sobel(image, axis=0, mode=mode, cval=cval)
-    imx = ndimage.sobel(image, axis=1, mode=mode, cval=cval)
+    imy = ndi.sobel(image, axis=0, mode=mode, cval=cval)
+    imx = ndi.sobel(image, axis=1, mode=mode, cval=cval)
 
     return imx, imy
 
@@ -54,7 +57,7 @@ def structure_tensor(image, sigma=1, mode='constant', cval=0):
     ----------
     image : ndarray
         Input image.
-    sigma : float
+    sigma : float, optional
         Standard deviation used for the Gaussian kernel, which is used as a
         weighting function for the local summation of squared differences.
     mode : {'constant', 'reflect', 'wrap', 'nearest', 'mirror'}, optional
@@ -92,20 +95,20 @@ def structure_tensor(image, sigma=1, mode='constant', cval=0):
     imx, imy = _compute_derivatives(image, mode=mode, cval=cval)
 
     # structure tensore
-    Axx = ndimage.gaussian_filter(imx * imx, sigma, mode=mode, cval=cval)
-    Axy = ndimage.gaussian_filter(imx * imy, sigma, mode=mode, cval=cval)
-    Ayy = ndimage.gaussian_filter(imy * imy, sigma, mode=mode, cval=cval)
+    Axx = ndi.gaussian_filter(imx * imx, sigma, mode=mode, cval=cval)
+    Axy = ndi.gaussian_filter(imx * imy, sigma, mode=mode, cval=cval)
+    Ayy = ndi.gaussian_filter(imy * imy, sigma, mode=mode, cval=cval)
 
     return Axx, Axy, Ayy
 
 
-def hessian_matrix(image, sigma=1, mode='constant', cval=0):
+def hessian_matrix(image, sigma=1, mode='constant', cval=0, order=None):
     """Compute Hessian matrix.
 
     The Hessian matrix is defined as::
 
-        H = [Hxx Hxy]
-            [Hxy Hyy]
+        H = [Hrr Hrc]
+            [Hrc Hcc]
 
     which is computed by convolving the image with the second derivatives
     of the Gaussian kernel in the respective x- and y-directions.
@@ -122,57 +125,64 @@ def hessian_matrix(image, sigma=1, mode='constant', cval=0):
     cval : float, optional
         Used in conjunction with mode 'constant', the value outside
         the image boundaries.
+    order : {'xy', 'rc'}, optional
+        This parameter allows for the use of reverse or forward order of
+        the image axes in gradient computation. 'xy' indicates the usage
+        of the last axis initially (Hxx, Hxy, Hyy), whilst 'rc' indicates
+        the use of the first axis initially (Hrr, Hrc, Hcc).
 
     Returns
     -------
-    Hxx : ndarray
+    Hrr : ndarray
         Element of the Hessian matrix for each pixel in the input image.
-    Hxy : ndarray
+    Hrc : ndarray
         Element of the Hessian matrix for each pixel in the input image.
-    Hyy : ndarray
+    Hcc : ndarray
         Element of the Hessian matrix for each pixel in the input image.
 
     Examples
     --------
     >>> from skimage.feature import hessian_matrix
     >>> square = np.zeros((5, 5))
-    >>> square[2, 2] = 1
-    >>> Hxx, Hxy, Hyy = hessian_matrix(square, sigma=0.1)
-    >>> Hxx
+    >>> square[2, 2] = 4
+    >>> Hrr, Hrc, Hcc = hessian_matrix(square, sigma=0.1, order = 'rc')
+    >>> Hrc
     array([[ 0.,  0.,  0.,  0.,  0.],
+           [ 0.,  1.,  0., -1.,  0.],
            [ 0.,  0.,  0.,  0.,  0.],
-           [ 0.,  0.,  1.,  0.,  0.],
-           [ 0.,  0.,  0.,  0.,  0.],
+           [ 0., -1.,  0.,  1.,  0.],
            [ 0.,  0.,  0.,  0.,  0.]])
-
     """
 
-    image = _prepare_grayscale_input_2D(image)
+    image = img_as_float(image)
 
-    # window extent to the left and right, which covers > 99% of the normal
-    # distribution
-    window_ext = max(1, np.ceil(3 * sigma))
+    gaussian_filtered = ndi.gaussian_filter(image, sigma=sigma,
+                                            mode=mode, cval=cval)
 
-    ky, kx = np.mgrid[-window_ext:window_ext + 1, -window_ext:window_ext + 1]
-
-    # second derivative Gaussian kernels
-    gaussian_exp = np.exp(-(kx ** 2 + ky ** 2) / (2 * sigma ** 2))
-    kernel_xx = 1 / (2 * np.pi * sigma ** 4) * (kx ** 2 / sigma ** 2 - 1)
-    kernel_xx *= gaussian_exp
-    kernel_xx /= kernel_xx.sum()
-    kernel_xy = 1 / (2 * np.pi * sigma ** 6) * (kx * ky)
-    kernel_xy *= gaussian_exp
-    kernel_xy /= kernel_xx.sum()
-    kernel_yy = kernel_xx.transpose()
-
-    Hxx = ndimage.convolve(image, kernel_xx, mode=mode, cval=cval)
-    Hxy = ndimage.convolve(image, kernel_xy, mode=mode, cval=cval)
-    Hyy = ndimage.convolve(image, kernel_yy, mode=mode, cval=cval)
-
-    return Hxx, Hxy, Hyy
+    if order is None:
+        if image.ndim == 2:
+            # The legacy 2D code followed (x, y) convention, so we swap the axis
+            # order to maintain compatibility with old code
+            warn('deprecation warning: the default order of the hessian matrix values '
+                 'will be "row-column" instead of "xy" starting in skimage version 0.15. '
+                 'Use order="rc" or order="xy" to set this explicitly')
+            order = 'xy'
+        else:
+            order = 'rc'
 
 
-def hessian_matrix_det(image, sigma):
+    gradients = np.gradient(gaussian_filtered)
+    axes = range(image.ndim)
+
+    if order == 'rc':
+        axes = reversed(axes)
+
+    H_elems = [np.gradient(gradients[ax0], axis=ax1)
+               for ax0, ax1 in combinations_with_replacement(axes, 2)]
+
+    return H_elems
+
+def hessian_matrix_det(image, sigma=1):
     """Computes the approximate Hessian Determinant over an image.
 
     This method uses box filters over integral images to compute the
@@ -182,7 +192,7 @@ def hessian_matrix_det(image, sigma):
     ----------
     image : array
         The image over which to compute Hessian Determinant.
-    sigma : float
+    sigma : float, optional
         Standard deviation used for the Gaussian kernel, used for the Hessian
         matrix.
 
@@ -255,7 +265,7 @@ def structure_tensor_eigvals(Axx, Axy, Ayy):
 
 
 def hessian_matrix_eigvals(Hxx, Hxy, Hyy):
-    """Compute Eigen values of Hessian matrix.
+    """Compute Eigenvalues of Hessian matrix.
 
     Parameters
     ----------
@@ -277,18 +287,90 @@ def hessian_matrix_eigvals(Hxx, Hxy, Hyy):
     --------
     >>> from skimage.feature import hessian_matrix, hessian_matrix_eigvals
     >>> square = np.zeros((5, 5))
-    >>> square[2, 2] = 1
-    >>> Hxx, Hxy, Hyy = hessian_matrix(square, sigma=0.1)
+    >>> square[2, 2] = 4
+    >>> Hxx, Hxy, Hyy = hessian_matrix(square, sigma=0.1, order='rc')
     >>> hessian_matrix_eigvals(Hxx, Hxy, Hyy)[0]
-    array([[ 0.,  0.,  0.,  0.,  0.],
-           [ 0.,  0.,  0.,  0.,  0.],
-           [ 0.,  0.,  1.,  0.,  0.],
-           [ 0.,  0.,  0.,  0.,  0.],
-           [ 0.,  0.,  0.,  0.,  0.]])
+    array([[ 0.,  0.,  2.,  0.,  0.],
+           [ 0.,  1.,  0.,  1.,  0.],
+           [ 2.,  0., -2.,  0.,  2.],
+           [ 0.,  1.,  0.,  1.,  0.],
+           [ 0.,  0.,  2.,  0.,  0.]])
 
     """
 
-    return _image_orthogonal_matrix22_eigvals(Hyy, Hxy, Hyy)
+    return _image_orthogonal_matrix22_eigvals(Hxx, Hxy, Hyy)
+
+
+def shape_index(image, sigma=1, mode='constant', cval=0):
+    """Compute the shape index.
+
+    The shape index, as defined by Koenderink & van Doorn [1]_, is a
+    single valued measure of local curvature, assuming the image as a 3D plane
+    with intensities representing heights.
+
+    It is derived from the eigen values of the Hessian, and its
+    value ranges from -1 to 1 (and is undefined (=NaN) in *flat* regions),
+    with following ranges representing following shapes:
+
+    .. table:: Ranges of the shape index and corresponding shapes.
+
+      ===================  =============
+      Interval (s in ...)  Shape
+      ===================  =============
+      [  -1, -7/8)         Spherical cup
+      [-7/8, -5/8)         Through
+      [-5/8, -3/8)         Rut
+      [-3/8, -1/8)         Saddle rut
+      [-1/8, +1/8)         Saddle
+      [+1/8, +3/8)         Saddle ridge
+      [+3/8, +5/8)         Ridge
+      [+5/8, +7/8)         Dome
+      [+7/8,   +1]         Spherical cap
+      ===================  =============
+
+    Parameters
+    ----------
+    image : ndarray
+        Input image.
+    sigma : float, optional
+        Standard deviation used for the Gaussian kernel, which is used for
+        smoothing the input data before Hessian eigen value calculation.
+    mode : {'constant', 'reflect', 'wrap', 'nearest', 'mirror'}, optional
+        How to handle values outside the image borders
+    cval : float, optional
+        Used in conjunction with mode 'constant', the value outside
+        the image boundaries.
+
+    Returns
+    -------
+    s : ndarray
+        Shape index
+
+    References
+    ----------
+    .. [1] Koenderink, J. J. & van Doorn, A. J.,
+           "Surface shape and curvature scales",
+           Image and Vision Computing, 1992, 10, 557-564.
+           DOI:10.1016/0262-8856(92)90076-F
+
+    Examples
+    --------
+    >>> from skimage.feature import shape_index
+    >>> square = np.zeros((5, 5))
+    >>> square[2, 2] = 4
+    >>> s = shape_index(square, sigma=0.1)
+    >>> s
+    array([[ nan,  nan, -0.5,  nan,  nan],
+           [ nan, -0. ,  nan, -0. ,  nan],
+           [-0.5,  nan, -1. ,  nan, -0.5],
+           [ nan, -0. ,  nan, -0. ,  nan],
+           [ nan,  nan, -0.5,  nan,  nan]])
+    """
+
+    Hxx, Hxy, Hyy = hessian_matrix(image, sigma=sigma, mode=mode, cval=cval, order='rc')
+    l1, l2 = hessian_matrix_eigvals(Hxx, Hxy, Hyy)
+
+    return (2.0 / np.pi) * np.arctan((l2 + l1) / (l2 - l1))
 
 
 def corner_kitchen_rosenfeld(image, mode='constant', cval=0):
@@ -323,8 +405,8 @@ def corner_kitchen_rosenfeld(image, mode='constant', cval=0):
     imxx, imxy = _compute_derivatives(imx, mode=mode, cval=cval)
     imyx, imyy = _compute_derivatives(imy, mode=mode, cval=cval)
 
-    numerator = (imxx * imy**2 + imyy * imx**2 - 2 * imxy * imx * imy)
-    denominator = (imx**2 + imy**2)
+    numerator = (imxx * imy ** 2 + imyy * imx ** 2 - 2 * imxy * imx * imy)
+    denominator = (imx ** 2 + imy ** 2)
 
     response = np.zeros_like(image, dtype=np.double)
 
@@ -403,12 +485,12 @@ def corner_harris(image, method='k', k=0.05, eps=1e-6, sigma=1):
     Axx, Axy, Ayy = structure_tensor(image, sigma)
 
     # determinant
-    detA = Axx * Ayy - Axy**2
+    detA = Axx * Ayy - Axy ** 2
     # trace
     traceA = Axx + Ayy
 
     if method == 'k':
-        response = detA - k * traceA**2
+        response = detA - k * traceA ** 2
     else:
         response = 2 * detA / (traceA + eps)
 
@@ -473,7 +555,7 @@ def corner_shi_tomasi(image, sigma=1):
     Axx, Axy, Ayy = structure_tensor(image, sigma)
 
     # minimum eigenvalue of A
-    response = ((Axx + Ayy) - np.sqrt((Axx - Ayy)**2 + 4 * Axy**2)) / 2
+    response = ((Axx + Ayy) - np.sqrt((Axx - Ayy) ** 2 + 4 * Axy ** 2)) / 2
 
     return response
 
@@ -543,7 +625,7 @@ def corner_foerstner(image, sigma=1):
     Axx, Axy, Ayy = structure_tensor(image, sigma)
 
     # determinant
-    detA = Axx * Ayy - Axy**2
+    detA = Axx * Ayy - Axy ** 2
     # trace
     traceA = Axx + Ayy
 
@@ -553,7 +635,7 @@ def corner_foerstner(image, sigma=1):
     mask = traceA != 0
 
     w[mask] = detA[mask] / traceA[mask]
-    q[mask] = 4 * detA[mask] / traceA[mask]**2
+    q[mask] = 4 * detA[mask] / traceA[mask] ** 2
 
     return w, q
 
@@ -680,7 +762,7 @@ def corner_subpix(image, corners, window_size=11, alpha=0.99):
     # window extent in one direction
     wext = (window_size - 1) // 2
 
-    image = pad(image, pad_width=wext, mode='constant', constant_values=0)
+    image = np.pad(image, pad_width=wext, mode='constant', constant_values=0)
 
     # add pad width, make sure to not modify the input values in-place
     corners = safe_as_int(corners + wext)
@@ -692,7 +774,7 @@ def corner_subpix(image, corners, window_size=11, alpha=0.99):
     b_edge = np.zeros((2, ), dtype=np.double)
 
     # critical statistical test values
-    redundancy = window_size**2 - 2
+    redundancy = window_size ** 2 - 2
     t_crit_dot = stats.f.isf(1 - alpha, redundancy, redundancy)
     t_crit_edge = stats.f.isf(alpha, redundancy, redundancy)
 
@@ -796,7 +878,7 @@ def corner_subpix(image, corners, window_size=11, alpha=0.99):
     return corners_subpix
 
 
-def corner_peaks(image, min_distance=10, threshold_abs=0, threshold_rel=0.1,
+def corner_peaks(image, min_distance=1, threshold_abs=None, threshold_rel=0.1,
                  exclude_border=True, indices=True, num_peaks=np.inf,
                  footprint=None, labels=None):
     """Find corners in corner measure response image.
@@ -820,18 +902,13 @@ def corner_peaks(image, min_distance=10, threshold_abs=0, threshold_rel=0.1,
            [ 0.,  0.,  1.,  1.,  0.],
            [ 0.,  0.,  1.,  1.,  0.],
            [ 0.,  0.,  0.,  0.,  0.]])
-    >>> peak_local_max(response, exclude_border=False)
-    array([[2, 2],
-           [2, 3],
+    >>> peak_local_max(response)
+    array([[3, 3],
            [3, 2],
-           [3, 3]])
-    >>> corner_peaks(response, exclude_border=False)
+           [2, 3],
+           [2, 2]])
+    >>> corner_peaks(response)
     array([[2, 2]])
-    >>> corner_peaks(response, exclude_border=False, min_distance=0)
-    array([[2, 2],
-           [2, 3],
-           [3, 2],
-           [3, 3]])
 
     """
 
@@ -853,3 +930,117 @@ def corner_peaks(image, min_distance=10, threshold_abs=0, threshold_rel=0.1,
         return np.transpose(peaks.nonzero())
     else:
         return peaks
+
+
+def corner_moravec(image, window_size=1):
+    """Compute Moravec corner measure response image.
+
+    This is one of the simplest corner detectors and is comparatively fast but
+    has several limitations (e.g. not rotation invariant).
+
+    Parameters
+    ----------
+    image : ndarray
+        Input image.
+    window_size : int, optional
+        Window size.
+
+    Returns
+    -------
+    response : ndarray
+        Moravec response image.
+
+    References
+    ----------
+    .. [1] http://kiwi.cs.dal.ca/~dparks/CornerDetection/moravec.htm
+    .. [2] http://en.wikipedia.org/wiki/Corner_detection
+
+    Examples
+    --------
+    >>> from skimage.feature import corner_moravec
+    >>> square = np.zeros([7, 7])
+    >>> square[3, 3] = 1
+    >>> square.astype(int)
+    array([[0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 1, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0]])
+    >>> corner_moravec(square).astype(int)
+    array([[0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 1, 1, 1, 0, 0],
+           [0, 0, 1, 2, 1, 0, 0],
+           [0, 0, 1, 1, 1, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0]])
+    """
+    return _corner_moravec(image, window_size)
+
+
+def corner_orientations(image, corners, mask):
+    """Compute the orientation of corners.
+
+    The orientation of corners is computed using the first order central moment
+    i.e. the center of mass approach. The corner orientation is the angle of
+    the vector from the corner coordinate to the intensity centroid in the
+    local neighborhood around the corner calculated using first order central
+    moment.
+
+    Parameters
+    ----------
+    image : 2D array
+        Input grayscale image.
+    corners : (N, 2) array
+        Corner coordinates as ``(row, col)``.
+    mask : 2D array
+        Mask defining the local neighborhood of the corner used for the
+        calculation of the central moment.
+
+    Returns
+    -------
+    orientations : (N, 1) array
+        Orientations of corners in the range [-pi, pi].
+
+    References
+    ----------
+    .. [1] Ethan Rublee, Vincent Rabaud, Kurt Konolige and Gary Bradski
+          "ORB : An efficient alternative to SIFT and SURF"
+          http://www.vision.cs.chubu.ac.jp/CV-R/pdf/Rublee_iccv2011.pdf
+    .. [2] Paul L. Rosin, "Measuring Corner Properties"
+          http://users.cs.cf.ac.uk/Paul.Rosin/corner2.pdf
+
+    Examples
+    --------
+    >>> from skimage.morphology import octagon
+    >>> from skimage.feature import (corner_fast, corner_peaks,
+    ...                              corner_orientations)
+    >>> square = np.zeros((12, 12))
+    >>> square[3:9, 3:9] = 1
+    >>> square.astype(int)
+    array([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+           [0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+           [0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+           [0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+           [0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+           [0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]])
+    >>> corners = corner_peaks(corner_fast(square, 9), min_distance=1)
+    >>> corners
+    array([[3, 3],
+           [3, 8],
+           [8, 3],
+           [8, 8]])
+    >>> orientations = corner_orientations(square, corners, octagon(3, 2))
+    >>> np.rad2deg(orientations)
+    array([  45.,  135.,  -45., -135.])
+
+    """
+    return _corner_orientations(image, corners, mask)
