@@ -1,5 +1,6 @@
 import numpy as np
-from numpy.testing import assert_equal, assert_almost_equal, run_module_suite
+from numpy.testing import (assert_equal, assert_almost_equal, run_module_suite,
+                           assert_raises)
 from skimage.feature import ORB
 from skimage import data
 from skimage._shared.testing import test_parallel
@@ -103,5 +104,11 @@ def test_descriptor_orb():
                  detector_extractor.descriptors[100:120, 10:20])
 
 
+def test_no_descriptors_extracted_orb():
+    img = np.ones((128, 128))
+    detector_extractor = ORB()
+    assert_raises(RuntimeError, detector_extractor.detect_and_extract, img)
+
+
 if __name__ == '__main__':
     run_module_suite()
