@@ -1,5 +1,11 @@
 import numpy as np
-from skimage.feature import greycomatrix, greycoprops, local_binary_pattern
+from skimage.feature import (greycomatrix,
+                             greycoprops,
+                             local_binary_pattern,
+                             multiblock_lbp)
+
+from skimage._shared.testing import test_parallel
+from skimage.transform import integral_image
 
 
 class TestGLCM():
@@ -10,6 +16,7 @@ class TestGLCM():
                                [0, 2, 2, 2],
                                [2, 2, 3, 3]], dtype=np.uint8)
 
+    @test_parallel()
     def test_output_angles(self):
         result = greycomatrix(self.image, [1], [0, np.pi / 4, np.pi / 2, 3 * np.pi / 4], 4)
         assert result.shape == (4, 4, 1, 4)
@@ -44,6 +51,34 @@ class TestGLCM():
                              [0, 0, 2, 0]], dtype=np.uint32)
         np.testing.assert_array_equal(result[:, :, 0, 0], expected)
 
+    def test_error_raise_float(self):
+        for dtype in [np.float, np.double, np.float16, np.float32, np.float64]:
+            np.testing.assert_raises(ValueError, greycomatrix, self.image.astype(dtype), [1], [np.pi], 4)
+
+    def test_error_raise_int_types(self):
+        for dtype in [np.int16, np.int32, np.int64, np.uint16, np.uint32, np.uint64]:
+            np.testing.assert_raises(ValueError, greycomatrix, self.image.astype(dtype), [1], [np.pi])
+
+    def test_error_raise_negative(self):
+        np.testing.assert_raises(ValueError, greycomatrix, self.image.astype(np.int16) - 1, [1], [np.pi], 4)
+
+    def test_error_raise_levels_smaller_max(self):
+        np.testing.assert_raises(ValueError, greycomatrix, self.image - 1, [1], [np.pi], 3)
+
+    def test_image_data_types(self):
+        for dtype in [np.uint16, np.uint32, np.uint64, np.int16, np.int32, np.int64]:
+            img = self.image.astype(dtype)
+            result = greycomatrix(img, [1], [np.pi / 2], 4,
+                                  symmetric=True)
+            assert result.shape == (4, 4, 1, 1)
+            expected = np.array([[6, 0, 2, 0],
+                                 [0, 4, 2, 0],
+                                 [2, 2, 2, 2],
+                                 [0, 0, 2, 0]], dtype=np.uint32)
+            np.testing.assert_array_equal(result[:, :, 0, 0], expected)
+
+        return
+
     def test_output_distance(self):
         im = np.array([[0, 0, 0, 0],
                        [1, 0, 0, 1],
@@ -162,6 +197,7 @@ class TestLBP():
                                [  0, 255,  30,  34,  255,  24],
                                [146, 241, 255,   0,  189, 126]], dtype='double')
 
+    @test_parallel()
     def test_default(self):
         lbp = local_binary_pattern(self.image, 8, 1, 'default')
         ref = np.array([[  0, 251,   0, 255,  96, 255],
@@ -199,7 +235,7 @@ class TestLBP():
         np.random.seed(13141516)
 
         # Create random image with known variance.
-        image = np.random.random((500, 500))
+        image = np.random.rand(500, 500)
         target_std = 0.3
         image = image / image.std() * target_std
 
@@ -208,7 +244,7 @@ class TestLBP():
         lbp = local_binary_pattern(image, P, R, 'var')
 
         # Take central part to avoid border effect.
-        lbp = lbp[5:-5,5:-5]
+        lbp = lbp[5:-5, 5:-5]
 
         # The LBP variance is biased (ddof=0), correct for that.
         expected = target_std**2 * (P-1)/P
@@ -226,5 +262,28 @@ class TestLBP():
         np.testing.assert_array_almost_equal(lbp, ref)
 
 
+class TestMBLBP():
+
+    def test_single_mblbp(self):
+
+        # Create dummy matrix where first and fifth rectangles have greater
+        # value than the central one. Therefore, the following bits
+        # should be 1.
+        test_img = np.zeros((9, 9), dtype='uint8')
+        test_img[3:6, 3:6] = 1
+        test_img[:3, :3] = 255
+        test_img[6:, 6:] = 255
+
+        # MB-LBP is filled in reverse order. So the first and fifth bits from
+        # the end should be filled.
+        correct_answer = 0b10001000
+
+        int_img = integral_image(test_img)
+
+        lbp_code = multiblock_lbp(int_img, 0, 0, 3, 3)
+
+        np.testing.assert_equal(lbp_code, correct_answer)
+
+
 if __name__ == '__main__':
     np.testing.run_module_suite()
