@@ -1,5 +1,5 @@
 from .graph_cut import cut_threshold, cut_normalized
-from .rag import rag_mean_color, RAG, draw_rag, rag_boundary
+from .rag import rag_mean_color, RAG, show_rag, rag_boundary
 from .graph_merge import merge_hierarchical
 ncut = cut_normalized
 
@@ -7,7 +7,7 @@ __all__ = ['rag_mean_color',
            'cut_threshold',
            'cut_normalized',
            'ncut',
-           'draw_rag',
+           'show_rag',
            'merge_hierarchical',
            'rag_boundary',
            'RAG']
