@@ -8,7 +8,8 @@ except ImportError:
     try:
         import pyfits
     except ImportError:
-        raise ImportError("PyFITS could not be found. Please refer to\n"
+        raise ImportError(
+            "PyFITS could not be found. Please refer to\n"
             "http://www.stsci.edu/resources/software_hardware/pyfits\n"
             "for further instructions.")
 
@@ -144,7 +145,7 @@ def FITSFactory(image_ext):
     hdulist.close()
 
     if data is None:
-        raise RuntimeError("Extension %d of %s has no data" %
-                  (extnum, filename))
+        raise RuntimeError(
+            "Extension %d of %s has no data" % (extnum, filename))
 
     return data
