@@ -5,6 +5,7 @@ from numpy.testing import assert_raises, assert_equal, assert_allclose
 
 from skimage import data_dir
 from skimage.io.collection import ImageCollection, alphanumeric_key
+from skimage.io import reset_plugins
 
 
 def test_string_split():
@@ -31,6 +32,7 @@ class TestImageCollection():
                        for pic in ['camera.png', 'moon.png']]
 
     def setUp(self):
+        reset_plugins()
         # Generic image collection with images of different shapes.
         self.images = ImageCollection(self.pattern)
         # Image collection with images having shapes that match.
@@ -78,6 +80,14 @@ class TestImageCollection():
         ic = ImageCollection(load_pattern, load_func=load_fn)
         assert_equal(ic[1], (2, 'two'))
 
+    def test_custom_load_func(self):
+
+        def load_fn(x):
+            return x
+
+        ic = ImageCollection(os.pathsep.join(self.pattern), load_func=load_fn)
+        assert_equal(ic[0], self.pattern[0])
+
     def test_concatenate(self):
         array = self.images_matched.concatenate()
         expected_shape = (len(self.images_matched),) + self.images[0].shape
