@@ -1,16 +1,19 @@
 from ._find_contours import find_contours
-from ._marching_cubes import (marching_cubes, mesh_surface_area,
-                              correct_mesh_orientation)
+from ._marching_cubes_lewiner import marching_cubes, marching_cubes_lewiner
+from ._marching_cubes_classic import (marching_cubes_classic,
+                                      mesh_surface_area,
+                                      correct_mesh_orientation)
 from ._regionprops import regionprops, perimeter
 from .simple_metrics import compare_mse, compare_nrmse, compare_psnr
 from ._structural_similarity import compare_ssim, structural_similarity
 from ._polygon import approximate_polygon, subdivide_polygon
-from ._pnpoly import points_in_poly, grid_points_in_poly
+from .pnpoly import points_in_poly, grid_points_in_poly
 from ._moments import moments, moments_central, moments_normalized, moments_hu
 from .profile import profile_line
 from .fit import LineModel, LineModelND, CircleModel, EllipseModel, ransac
 from .block import block_reduce
 from ._label import label
+from .entropy import shannon_entropy
 
 
 __all__ = ['find_contours',
@@ -29,6 +32,8 @@ __all__ = ['find_contours',
            'moments_normalized',
            'moments_hu',
            'marching_cubes',
+           'marching_cubes_lewiner',
+           'marching_cubes_classic',
            'mesh_surface_area',
            'correct_mesh_orientation',
            'profile_line',
@@ -40,4 +45,5 @@ __all__ = ['find_contours',
            'compare_mse',
            'compare_nrmse',
            'compare_psnr',
+           'shannon_entropy',
            ]
