@@ -1,5 +1,5 @@
 import numpy as np
-from skimage.util import view_as_blocks, pad
+from ..util import view_as_blocks
 
 
 def block_reduce(image, block_size, func=np.sum, cval=0):
@@ -13,8 +13,8 @@ def block_reduce(image, block_size, func=np.sum, cval=0):
         Array containing down-sampling integer factor along each axis.
     func : callable
         Function object which is used to calculate the return value for each
-        local block. This function must implement an ``axis`` parameter such as
-        ``numpy.sum`` or ``numpy.min``.
+        local block. This function must implement an ``axis`` parameter such
+        as ``numpy.sum`` or ``numpy.min``.
     cval : float
         Constant padding value if image is not perfectly divisible by the
         block size.
@@ -58,14 +58,18 @@ def block_reduce(image, block_size, func=np.sum, cval=0):
 
     pad_width = []
     for i in range(len(block_size)):
+        if block_size[i] < 1:
+            raise ValueError("Down-sampling factors must be >= 1. Use "
+                             "`skimage.transform.resize` to up-sample an "
+                             "image.")
         if image.shape[i] % block_size[i] != 0:
             after_width = block_size[i] - (image.shape[i] % block_size[i])
         else:
             after_width = 0
         pad_width.append((0, after_width))
 
-    image = pad(image, pad_width=pad_width, mode='constant',
-                constant_values=cval)
+    image = np.pad(image, pad_width=pad_width, mode='constant',
+                   constant_values=cval)
 
     out = view_as_blocks(image, block_size)
 
