@@ -4,15 +4,14 @@ from .grey import (erosion, dilation, opening, closing, white_tophat,
                    black_tophat)
 from .selem import (square, rectangle, diamond, disk, cube, octahedron, ball,
                     octagon, star)
+from ..measure._label import label
 from .watershed import watershed
-from ._skeletonize import skeletonize, medial_axis
+from ._skeletonize import skeletonize, medial_axis, thin
+from ._skeletonize_3d import skeletonize_3d
 from .convex_hull import convex_hull_image, convex_hull_object
 from .greyreconstruct import reconstruction
-from .misc import remove_small_objects
-
-from ..measure._label import label
-from skimage._shared.utils import deprecated as _deprecated
-label = _deprecated('skimage.measure.label')(label)
+from .misc import remove_small_objects, remove_small_holes
+from .extrema import (h_minima, h_maxima, local_maxima, local_minima)
 
 
 __all__ = ['binary_erosion',
@@ -33,11 +32,19 @@ __all__ = ['binary_erosion',
            'octahedron',
            'ball',
            'octagon',
+           'star',
            'label',
            'watershed',
            'skeletonize',
+           'skeletonize_3d',
+           'thin',
            'medial_axis',
            'convex_hull_image',
            'convex_hull_object',
            'reconstruction',
-           'remove_small_objects']
+           'remove_small_objects',
+           'remove_small_holes',
+           'h_minima',
+           'h_maxima',
+           'local_maxima',
+           'local_minima']
