@@ -2,10 +2,16 @@
 Algorithms for computing the skeleton of a binary image
 """
 
+import sys
+from six.moves import range
 import numpy as np
-from scipy import ndimage
+from scipy import ndimage as ndi
+
+from ._skeletonize_cy import (_fast_skeletonize, _skeletonize_loop,
+                              _table_lookup_index)
+
+from .._shared.utils import assert_nD
 
-from ._skeletonize_cy import _skeletonize_loop, _table_lookup_index
 
 # --------- Skeletonization by morphological thinning ---------
 
@@ -34,7 +40,7 @@ def skeletonize(image):
 
     Notes
     -----
-    The algorithm [1] works by making successive passes of the image,
+    The algorithm [Zha84]_ works by making successive passes of the image,
     removing pixels on object borders. This continues until no
     more pixels can be removed.  The image is correlated with a
     mask that assigns each pixel a number in the range [0...255]
@@ -49,9 +55,9 @@ def skeletonize(image):
 
     References
     ----------
-    .. [1] A fast parallel algorithm for thinning digital patterns,
-       T. Y. ZHANG and C. Y. SUEN, Communications of the ACM,
-       March 1984, Volume 27, Number 3
+    .. [Zha84] A fast parallel algorithm for thinning digital patterns,
+           T. Y. Zhang and C. Y. Suen, Communications of the ACM,
+           March 1984, Volume 27, Number 3.
 
 
     Examples
@@ -81,80 +87,205 @@ def skeletonize(image):
            [0, 0, 0, 0, 0, 0, 0, 0, 0]], dtype=uint8)
 
     """
-    # look up table - there is one entry for each of the 2^8=256 possible
-    # combinations of 8 binary neighbours. 1's, 2's and 3's are candidates
-    # for removal at each iteration of the algorithm.
-    lut = [0, 0, 0, 1, 0, 0, 1, 3, 0, 0, 3, 1, 1, 0, 1, 3, 0, 0, 0, 0, 0, 0,
-           0, 0, 2, 0, 2, 0, 3, 0, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0,
-           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 3, 0, 2, 2, 0, 0,
-           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-           0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 2, 0,
-           0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 3, 0, 2, 0, 0, 0, 3, 1,
-           0, 0, 1, 3, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-           0, 0, 0, 0, 0, 1, 3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-           2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 3, 1, 3, 0, 0,
-           1, 3, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-           0, 0, 0, 0, 2, 3, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 3, 3,
-           0, 1, 0, 0, 0, 0, 2, 2, 0, 0, 2, 0, 0, 0]
 
     # convert to unsigned int (this should work for boolean values)
-    skeleton = image.astype(np.uint8)
+    image = image.astype(np.uint8)
 
     # check some properties of the input image:
     #  - 2D
     #  - binary image with only 0's and 1's
-    if skeleton.ndim != 2:
+    if image.ndim != 2:
         raise ValueError('Skeletonize requires a 2D array')
-    if not np.all(np.in1d(skeleton.flat, (0, 1))):
+    if not np.all(np.in1d(image.flat, (0, 1))):
         raise ValueError('Image contains values other than 0 and 1')
 
-    # create the mask that will assign a unique value based on the
-    #  arrangement of neighbouring pixels
-    mask = np.array([[  1,  2,  4],
-                     [128,  0,  8],
-                     [ 64, 32, 16]], np.uint8)
+    return _fast_skeletonize(image)
 
-    pixel_removed = True
-    while pixel_removed:
-        pixel_removed = False
 
-        # assign each pixel a unique value based on its foreground neighbours
-        neighbours = ndimage.correlate(skeleton, mask, mode='constant')
+# --------- Skeletonization and thinning based on Guo and Hall 1989 ---------
 
-        # ignore background
-        neighbours *= skeleton
+def _generate_thin_luts():
+    """generate LUTs for thinning algorithm (for reference)"""
 
-        # use LUT to categorize each foreground pixel as a 0, 1, 2 or 3
-        codes = np.take(lut, neighbours)
+    def nabe(n):
+        return np.array([n >> i & 1 for i in range(0, 9)]).astype(np.bool)
 
-        # pass 1 - remove the 1's and 3's
-        code_mask = (codes == 1)
-        if np.any(code_mask):
-            pixel_removed = True
-            skeleton[code_mask] = 0
-        code_mask = (codes == 3)
-        if np.any(code_mask):
-            pixel_removed = True
-            skeleton[code_mask] = 0
+    def G1(n):
+        s = 0
+        bits = nabe(n)
+        for i in (0, 2, 4, 6):
+            if not(bits[i]) and (bits[i + 1] or bits[(i + 2) % 8]):
+                s += 1
+        return s == 1
 
-        # pass 2 - remove the 2's and 3's
-        neighbours = ndimage.correlate(skeleton, mask, mode='constant')
-        neighbours *= skeleton
-        codes = np.take(lut, neighbours)
-        code_mask = (codes == 2)
-        if np.any(code_mask):
-            pixel_removed = True
-            skeleton[code_mask] = 0
-        code_mask = (codes == 3)
-        if np.any(code_mask):
-            pixel_removed = True
-            skeleton[code_mask] = 0
+    g1_lut = np.array([G1(n) for n in range(256)])
+
+    def G2(n):
+        n1, n2 = 0, 0
+        bits = nabe(n)
+        for k in (1, 3, 5, 7):
+            if bits[k] or bits[k - 1]:
+                n1 += 1
+            if bits[k] or bits[(k + 1) % 8]:
+                n2 += 1
+        return min(n1, n2) in [2, 3]
+
+    g2_lut = np.array([G2(n) for n in range(256)])
+
+    g12_lut = g1_lut & g2_lut
+
+    def G3(n):
+        bits = nabe(n)
+        return not((bits[1] or bits[2] or not(bits[7])) and bits[0])
+
+    def G3p(n):
+        bits = nabe(n)
+        return not((bits[5] or bits[6] or not(bits[3])) and bits[4])
+
+    g3_lut = np.array([G3(n) for n in range(256)])
+    g3p_lut = np.array([G3p(n) for n in range(256)])
+
+    g123_lut = g12_lut & g3_lut
+    g123p_lut = g12_lut & g3p_lut
+
+    return g123_lut, g123p_lut
+
+
+G123_LUT = np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,
+                     0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0,
+                     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1,
+                     0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                     0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1,
+                     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
+                     0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0,
+                     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                     0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
+                     1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0,
+                     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0,
+                     0, 1, 1, 0, 0, 1, 0, 0, 0], dtype=np.bool)
+
+G123P_LUT = np.array([0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0,
+                      0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,
+                      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                      0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0,
+                      0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                      0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
+                      1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0,
+                      0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                      0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0,
+                      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1,
+                      0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                      0, 0, 0, 0, 0, 0, 0, 0, 0], dtype=np.bool)
+
+
+def thin(image, max_iter=None):
+    """
+    Perform morphological thinning of a binary image.
+
+    Parameters
+    ----------
+    image : binary (M, N) ndarray
+        The image to be thinned.
+
+    max_iter : int, number of iterations, optional
+        Regardless of the value of this parameter, the thinned image
+        is returned immediately if an iteration produces no change.
+        If this parameter is specified it thus sets an upper bound on
+        the number of iterations performed.
+
+    Returns
+    -------
+    out : ndarray of bool
+        Thinned image.
+
+    See also
+    --------
+    skeletonize, skeletonize_3d, medial_axis
+
+    Notes
+    -----
+    This algorithm [1]_ works by making multiple passes over the image,
+    removing pixels matching a set of criteria designed to thin
+    connected regions while preserving eight-connected components and
+    2 x 2 squares [2]_. In each of the two sub-iterations the algorithm
+    correlates the intermediate skeleton image with a neighborhood mask,
+    then looks up each neighborhood in a lookup table indicating whether
+    the central pixel should be deleted in that sub-iteration.
+
+    References
+    ----------
+    .. [1] Z. Guo and R. W. Hall, "Parallel thinning with
+           two-subiteration algorithms," Comm. ACM, vol. 32, no. 3,
+           pp. 359-373, 1989. DOI:10.1145/62065.62074
+    .. [2] Lam, L., Seong-Whan Lee, and Ching Y. Suen, "Thinning
+           Methodologies-A Comprehensive Survey," IEEE Transactions on
+           Pattern Analysis and Machine Intelligence, Vol 14, No. 9,
+           p. 879, 1992. DOI:10.1109/34.161346
+
+    Examples
+    --------
+    >>> square = np.zeros((7, 7), dtype=np.uint8)
+    >>> square[1:-1, 2:-2] = 1
+    >>> square[0, 1] =  1
+    >>> square
+    array([[0, 1, 0, 0, 0, 0, 0],
+           [0, 0, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0]], dtype=uint8)
+    >>> skel = thin(square)
+    >>> skel.astype(np.uint8)
+    array([[0, 1, 0, 0, 0, 0, 0],
+           [0, 0, 1, 0, 0, 0, 0],
+           [0, 0, 0, 1, 0, 0, 0],
+           [0, 0, 0, 1, 0, 0, 0],
+           [0, 0, 0, 1, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0]], dtype=uint8)
+    """
+    # check parameters
+    max_iter = max_iter or sys.maxsize
+    # check that image is 2d
+    assert_nD(image, 2)
+
+    # convert image to uint8 with values in {0, 1}
+    skel = np.asanyarray(image, dtype=bool).astype(np.uint8)
+
+    # neighborhood mask
+    mask = np.array([[ 8,  4,  2],
+                     [16,  0,  1],
+                     [32, 64,128]], dtype=np.uint8)
+
+    # iterate until convergence, up to the iteration limit
+    for i in range(max_iter):
+        before = np.sum(skel)  # count points before thinning
+        # perform the two "subiterations" described in the paper
+        for lut in [G123_LUT, G123P_LUT]:
+            # correlate image with neighborhood mask
+            N = ndi.correlate(skel, mask, mode='constant')
+            # take deletion decision from this subiteration's LUT
+            D = np.take(lut, N)
+            # perform deletion
+            skel[D] = 0
+
+        after = np.sum(skel)  # count points after thinning
+
+        if before == after:
+            # iteration had no effect: finish
+            break
+
+    return skel.astype(np.bool)
 
-    return skeleton.astype(bool)
 
 # --------- Skeletonization by medial axis transform --------
 
-_eight_connect = ndimage.generate_binary_structure(2, 2)
+_eight_connect = ndi.generate_binary_structure(2, 2)
 
 
 def medial_axis(image, mask=None, return_distance=False):
@@ -163,23 +294,19 @@ def medial_axis(image, mask=None, return_distance=False):
 
     Parameters
     ----------
-
-    image : binary ndarray
-
-    mask : binary ndarray, optional
-        If a mask is given, only those elements with a true value in `mask`
-        are used for computing the medial axis.
-
+    image : binary ndarray, shape (M, N)
+        The image of the shape to be skeletonized.
+    mask : binary ndarray, shape (M, N), optional
+        If a mask is given, only those elements in `image` with a true
+        value in `mask` are used for computing the medial axis.
     return_distance : bool, optional
         If true, the distance transform is returned as well as the skeleton.
 
     Returns
     -------
-
     out : ndarray of bools
         Medial axis transform of the image
-
-    dist : ndarray of ints
+    dist : ndarray of ints, optional
         Distance transform of the image (only returned if `return_distance`
         is True)
 
@@ -251,17 +378,17 @@ def medial_axis(image, mask=None, return_distance=False):
     center_is_foreground = (np.arange(512) & 2**4).astype(bool)
     table = (center_is_foreground  # condition 1.
                 &
-            (np.array([ndimage.label(_pattern_of(index), _eight_connect)[1] !=
-                        ndimage.label(_pattern_of(index & ~ 2**4),
+            (np.array([ndi.label(_pattern_of(index), _eight_connect)[1] !=
+                       ndi.label(_pattern_of(index & ~ 2**4),
                                     _eight_connect)[1]
-                        for index in range(512)])  # condition 2
+                       for index in range(512)])  # condition 2
                 |
         np.array([np.sum(_pattern_of(index)) < 3 for index in range(512)]))
         # condition 3
             )
 
     # Build distance transform
-    distance = ndimage.distance_transform_edt(masked_image)
+    distance = ndi.distance_transform_edt(masked_image)
     if return_distance:
         store_distance = distance.copy()
 
