@@ -35,7 +35,7 @@ if __name__ == '__main__':
     from numpy.distutils.core import setup
     setup(maintainer='scikit-image Developers',
           author='Damian Eads',
-          maintainer_email='scikit-image@googlegroups.com',
+          maintainer_email='scikit-image@python.org',
           description='Morphology Wrapper',
           url='https://github.com/scikit-image/scikit-image',
           license='SciPy License (BSD Style)',
