@@ -39,7 +39,7 @@ if __name__ == '__main__':
     from numpy.distutils.core import setup
     setup(maintainer='scikit-image Developers',
           author='scikit-image Developers',
-          maintainer_email='scikit-image@googlegroups.com',
+          maintainer_email='scikit-image@python.org',
           description='Restoration',
           url='https://github.com/scikit-image/scikit-image',
           license='SciPy License (BSD Style)',
