@@ -1,13 +1,16 @@
 from .random_walker_segmentation import random_walker
+from .active_contour_model import active_contour
 from ._felzenszwalb import felzenszwalb
 from .slic_superpixels import slic
 from ._quickshift import quickshift
 from .boundaries import find_boundaries, mark_boundaries
 from ._clear_border import clear_border
 from ._join import join_segmentations, relabel_from_one, relabel_sequential
+from ..morphology import watershed
 
 
 __all__ = ['random_walker',
+           'active_contour',
            'felzenszwalb',
            'slic',
            'quickshift',
@@ -16,4 +19,5 @@ __all__ = ['random_walker',
            'clear_border',
            'join_segmentations',
            'relabel_from_one',
-           'relabel_sequential']
+           'relabel_sequential',
+           'watershed']
