@@ -1,10 +1,11 @@
 import itertools as it
-import warnings
 import numpy as np
 from numpy.testing import assert_equal, assert_raises
 from skimage.segmentation import slic
+from skimage._shared.testing import test_parallel
 
 
+@test_parallel()
 def test_color_2d():
     rnd = np.random.RandomState(0)
     img = np.zeros((20, 21, 3))
@@ -14,9 +15,27 @@ def test_color_2d():
     img += 0.01 * rnd.normal(size=img.shape)
     img[img > 1] = 1
     img[img < 0] = 0
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore")
-        seg = slic(img, n_segments=4, sigma=0)
+    seg = slic(img, n_segments=4, sigma=0, enforce_connectivity=False)
+
+    # we expect 4 segments
+    assert_equal(len(np.unique(seg)), 4)
+    assert_equal(seg.shape, img.shape[:-1])
+    assert_equal(seg[:10, :10], 0)
+    assert_equal(seg[10:, :10], 2)
+    assert_equal(seg[:10, 10:], 1)
+    assert_equal(seg[10:, 10:], 3)
+
+
+def test_multichannel_2d():
+    rnd = np.random.RandomState(0)
+    img = np.zeros((20, 20, 8))
+    img[:10, :10, 0:2] = 1
+    img[:10, 10:, 2:4] = 1
+    img[10:, :10, 4:6] = 1
+    img[10:, 10:, 6:8] = 1
+    img += 0.01 * rnd.normal(size=img.shape)
+    img = np.clip(img, 0, 1, out=img)
+    seg = slic(img, n_segments=4, enforce_connectivity=False)
 
     # we expect 4 segments
     assert_equal(len(np.unique(seg)), 4)
@@ -136,6 +155,12 @@ def test_enforce_connectivity():
                                  enforce_connectivity=False,
                                  convert2lab=False)
 
+    # Make sure nothing fatal occurs (e.g. buffer overflow) at low values of
+    # max_size_factor
+    segments_connected_low_max = slic(img, 2, compactness=0.0001,
+                                      enforce_connectivity=True,
+                                      convert2lab=False, max_size_factor=0.8)
+
     result_connected = np.array([[0, 0, 0, 1, 1, 1],
                                  [0, 0, 0, 1, 1, 1],
                                  [0, 0, 0, 1, 1, 1]], np.float)
@@ -146,6 +171,7 @@ def test_enforce_connectivity():
 
     assert_equal(segments_connected, result_connected)
     assert_equal(segments_disconnected, result_disconnected)
+    assert_equal(segments_connected_low_max, result_connected)
 
 
 def test_slic_zero():
@@ -158,9 +184,7 @@ def test_slic_zero():
     img += 0.01 * rnd.normal(size=img.shape)
     img[img > 1] = 1
     img[img < 0] = 0
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore")
-        seg = slic(img, n_segments=4, sigma=0, slic_zero=True)
+    seg = slic(img, n_segments=4, sigma=0, slic_zero=True)
 
     # we expect 4 segments
     assert_equal(len(np.unique(seg)), 4)
@@ -171,6 +195,20 @@ def test_slic_zero():
     assert_equal(seg[10:, 10:], 3)
 
 
+def test_more_segments_than_pixels():
+    rnd = np.random.RandomState(0)
+    img = np.zeros((20, 21))
+    img[:10, :10] = 0.33
+    img[10:, :10] = 0.67
+    img[10:, 10:] = 1.00
+    img += 0.0033 * rnd.normal(size=img.shape)
+    img[img > 1] = 1
+    img[img < 0] = 0
+    seg = slic(img, sigma=0, n_segments=500, compactness=1,
+               multichannel=False, convert2lab=False)
+    assert np.all(seg.ravel() == np.arange(seg.size))
+
+
 if __name__ == '__main__':
     from numpy import testing
 
