@@ -2,10 +2,17 @@ from .dtype import (img_as_float, img_as_int, img_as_uint, img_as_ubyte,
                     img_as_bool, dtype_limits)
 from .shape import view_as_blocks, view_as_windows
 from .noise import random_noise
+from .apply_parallel import apply_parallel
 
-from .arraypad import pad
-from ._regular_grid import regular_grid
+from .arraycrop import crop
+from ._regular_grid import regular_grid, regular_seeds
 from .unique import unique_rows
+from ._invert import invert
+
+from .._shared.utils import copy_func
+
+from numpy import pad as numpy_pad
+pad = copy_func(numpy_pad, name='pad')
 
 
 __all__ = ['img_as_float',
@@ -17,6 +24,10 @@ __all__ = ['img_as_float',
            'view_as_blocks',
            'view_as_windows',
            'pad',
+           'crop',
            'random_noise',
            'regular_grid',
+           'regular_seeds',
+           'apply_parallel',
+           'invert',
            'unique_rows']
