@@ -2,9 +2,8 @@ import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.colors as mcolors
 LABELS_CMAP = mcolors.ListedColormap(['white', 'red', 'dodgerblue', 'gold',
-                                          'greenyellow', 'blueviolet'])
-
-from skimage.viewer.canvastools.base import CanvasToolBase
+                                      'greenyellow', 'blueviolet'])
+from ...viewer.canvastools.base import CanvasToolBase
 
 
 __all__ = ['PaintTool']
@@ -15,8 +14,8 @@ class PaintTool(CanvasToolBase):
 
     Parameters
     ----------
-    ax : :class:`matplotlib.axes.Axes`
-        Matplotlib axes where tool is displayed.
+    manager : Viewer or PlotPlugin.
+        Skimage viewer or plot plugin object.
     overlay_shape : shape tuple
         2D shape tuple used to initialize overlay image.
     alpha : float (between [0, 1])
@@ -38,10 +37,30 @@ class PaintTool(CanvasToolBase):
         Overlay of painted labels displayed on top of image.
     label : int
         Current paint color.
+
+    Examples
+    ----------
+    >>> from skimage.data import camera
+    >>> import matplotlib.pyplot as plt
+    >>> from skimage.viewer.canvastools import PaintTool
+    >>> import numpy as np
+
+    >>> img = camera() #doctest: +SKIP
+
+    >>> ax = plt.subplot(111) #doctest: +SKIP 
+    >>> plt.imshow(img, cmap=plt.cm.gray) #doctest: +SKIP
+    >>> p = PaintTool(ax,np.shape(img[:-1]),10,0.2) #doctest: +SKIP
+    >>> plt.show() #doctest: +SKIP
+
+    >>> mask = p.overlay #doctest: +SKIP
+    >>> plt.imshow(mask,cmap=plt.cm.gray) #doctest: +SKIP
+    >>> plt.show() #doctest: +SKIP
     """
-    def __init__(self, ax, overlay_shape, radius=5, alpha=0.3, on_move=None,
-                 on_release=None, on_enter=None, rect_props=None):
-        super(PaintTool, self).__init__(ax, on_move=on_move, on_enter=on_enter,
+    def __init__(self, manager, overlay_shape, radius=5, alpha=0.3,
+                 on_move=None, on_release=None, on_enter=None,
+                 rect_props=None):
+        super(PaintTool, self).__init__(manager, on_move=on_move,
+                                        on_enter=on_enter,
                                         on_release=on_release)
 
         props = dict(edgecolor='r', facecolor='0.7', alpha=0.5, animated=True)
@@ -61,11 +80,8 @@ class PaintTool(CanvasToolBase):
         self.radius = radius
 
         # Note that the order is important: Redraw cursor *after* overlay
-        self._artists = [self._overlay_plot, self._cursor]
-
-        self.connect_event('button_press_event', self.on_mouse_press)
-        self.connect_event('button_release_event', self.on_mouse_release)
-        self.connect_event('motion_notify_event', self.on_move)
+        self.artists = [self._overlay_plot, self._cursor]
+        self.manager.add_tool(self)
 
     @property
     def label(self):
@@ -102,7 +118,7 @@ class PaintTool(CanvasToolBase):
             self._overlay_plot = None
         elif self._overlay_plot is None:
             props = dict(cmap=self.cmap, alpha=self.alpha,
-                         norm=mcolors.no_norm(), animated=True)
+                         norm=mcolors.NoNorm(), animated=True)
             self._overlay_plot = self.ax.imshow(image, **props)
         else:
             self._overlay_plot.set_data(image)
@@ -121,7 +137,7 @@ class PaintTool(CanvasToolBase):
             self.radius = self._radius
         self.overlay = np.zeros(shape, dtype='uint8')
 
-    def _on_key_press(self, event):
+    def on_key_press(self, event):
         if event.key == 'enter':
             self.callback_on_enter(self.geometry)
             self.redraw()
@@ -198,13 +214,13 @@ class CenteredWindow(object):
         return [slice(ymin, ymax), slice(xmin, xmax)]
 
 
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     np.testing.rundocs()
-    from skimage import data
+    from ... import data
+    from ...viewer import ImageViewer
 
     image = data.camera()
 
-    f, ax = plt.subplots()
-    ax.imshow(image, interpolation='nearest')
-    paint_tool = PaintTool(ax, image.shape)
-    plt.show()
+    viewer = ImageViewer(image)
+    paint_tool = PaintTool(viewer, image.shape)
+    viewer.show()
