@@ -23,7 +23,7 @@ from skimage.viewer import CollectionViewer
 from skimage.transform import pyramid_gaussian
 
 
-img = data.lena()
+img = data.astronaut()
 img_collection = tuple(pyramid_gaussian(img))
 
 view = CollectionViewer(img_collection)
